library(maptools);library(raster);library(spatialEco);library(fields);library(MASS)
library(MCMCpack);library(cowplot);library(ggplot2);library(CompGLM);library(spdep)
library(maptools);library(rgdal);library(sf)

#If cannot install "CompGLM", then try either one of the following
# install.packages("~/Downloads/CompGLM_1.0.tar", repos = NULL, type="source")
##Or
# library(devtools)
# install_github("jeffpollock9/CompGLM")
# library(CompGLM)


##Load Data
load('VotingData.RData')

#Pre-setting
n.iter=30000
lambda=matrix(1,n,n.iter)
nu=matrix(1,n,n.iter)
for(g in 1:n){
  pdf<-function(lambda1, nu1){
    normalize=1
    # for(j in 1:truncate.number){
    #   normalize=normalize+(lambda1^truncate.number)/(factorial(truncate.number)^nu1)
    # }
    a=Z[g]+2                  #a0
    b=lfactorial(Z[g])+2      #b0; lfactorial(20)  log(factorial(Z[g]))
    c=1+35                    #c0;
    loglikeli=(a-1)*log(lambda1)-nu1*b-c*log(normalize)
    likeli=exp(loglikeli)
    return(likeli)
  }
  demo=multi_slice_sampling(n.iter,pdf = pdf,x0=c(0.9,1.05),width=c(0.055,0.0005)) #c(0.85,1.1) use this
  lambda[g,]=demo[,1]
  nu[g,]=demo[,2]
  
  print(g)
}

###MCMC
#Burn in
burn_in=20000

#####Setting
beta1.star = matrix(1,n,(n.iter))
beta2.star = matrix(1,n,(n.iter)) 
sig2.lambda=matrix(1,1,(n.iter))
sig2.nu=matrix(1,1,(n.iter)) 

####
Xpart=X%*%solve(t(X)%*%X)%*%t(X)
Psipart=Psi%*%ginv(t(Psi)%*%Psi)%*%t(Psi)

for(b in 2:(n.iter)){
  ###update beta1.star
  ############################################
  mu.lambda=Xpart%*%log(lambda[,(b)])+Psipart%*%(log(lambda[,(b)])-Xpart%*%log(lambda[,(b)]))
  beta1.star[,b]=mu.lambda+rnorm(n,0,1)*sqrt(sig2.lambda[b-1])
  
  ###update beta2.star
  ############################################
  mu.nu=Xpart%*%log(nu[,(b)])+Psipart%*%(log(nu[,(b)])-Xpart%*%log(nu[,(b)]))
  #mu.nu=Xpart%*%log(nu[,(burn_in+b)])
  beta2.star[,b]=mu.nu+rnorm(n,0,1)*sqrt(sig2.nu[b-1])
  
  ###update sig2.lambda
  ############################################
  sig2.lambda[b]=rinvgamma(1,shape =(n)/2+20 ,scale = 0.1+t(beta1.star[,b]-mu.lambda)%*%(beta1.star[,b]-mu.lambda)/2)
  
  ###update sig2.nu
  ############################################
  sig2.nu[b]=rinvgamma(1,shape =(n)/2+100 ,scale = 0.1+t(beta2.star[,b]-mu.nu)%*%(beta2.star[,b]-mu.nu)/2)
  
  ###########################################################################################################
  if ((b%%5000) == 0){print(b)}
}

###Plot and Posterior results
Zhat=CountyData
Zhat[,2:(n.iter-burn_in+1)]=0
for(j in (burn_in+1):n.iter){
  for(i in 1:length(Z)){
    Zhat[i,(j-burn_in+1)]=rcomp(1,lam = exp(beta1.star[,j])[i], nu = exp(beta2.star[,j])[i], sumTo = 100L) 
  }
  if ((j%%500) == 0){print(j)}
}

Zhat$Pred.Vote=apply(Zhat[,2:(n.iter-burn_in)],1,mean)
Zhat=Zhat[,c(1,(n.iter-burn_in+2))]
trydata=merge(trydata,Zhat)

rng= range(c(min(trydata$Vote.differ), max(ceiling(c(max(trydata$Vote.differ),max(trydata$Pred.Vote))))))
#OURS Plot
ggplot(trydata) +
  geom_sf(aes(fill=Pred.Vote)) +
  ggtitle("Posterior Mean (OURS)")+
  labs(fill="") +
  scale_fill_continuous(limits=c(floor(rng[1]), ceiling(rng[2])))


